import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:ky_tool_plugin/http/dio_utils.dart';
import 'package:ky_tool_plugin/http/http_utils.dart';
import 'package:ky_tool_plugin/tools/ky_log_utils.dart';
import 'package:ky_tool_plugin/tools/ky_toast.dart';
import 'package:package_info/package_info.dart';
import 'package:url_launcher/url_launcher.dart';

class KYVersionUpdateManager{


  // 获取版本号以及是否强制更新
  void getNewerVersionAndIsForceUpdate(bool isShowTip,BuildContext context,String requestUrl) async{

    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    String currentVersion = packageInfo.version;

    bool isAndroid = false;

    if (Platform.isAndroid){
      print('run android');
      isAndroid = true;
    }


    String newVersion = '';
    bool isForceUpdate = false;
    String andriodDownloadUrl = 'https://www.pgyer.com/qOmV';
    String iosDownloadUrl = 'https://www.pgyer.com/WThF';
    HttpUtils.request(Method.GET, requestUrl, {}, success: (result){
      LogUtils.d('getNewVersionAndIsForceUpdate====$result');
      if (result['code'] == 0){
        if (result['data'] is Map){
          Map dataMap = result['data'];
          String versionKey;
          if (isAndroid){
            versionKey = 'version';
            if (dataMap.keys.contains('andriod_download')){
              if (dataMap['andriod_download'] is String){
                andriodDownloadUrl = dataMap['andriod_download'];
              }
            }
          }else{
            versionKey = 'ios_version';
            if (dataMap.keys.contains('ios_download')){
              if (dataMap['ios_download'] is String){
                iosDownloadUrl = dataMap['ios_download'];
              }
            }
          }
          if (dataMap.keys.contains(versionKey)){
            if (dataMap[versionKey] is String){
              String versionStr = dataMap[versionKey];
              newVersion = versionStr.replaceAll('v', '').replaceAll('V', '');
            }
          }
          if (dataMap.keys.contains('forced_update')){
            if (dataMap['forced_update'] is String){
              if (int.parse(dataMap['forced_update']) == 1){
                isForceUpdate = true;
              }else{
                isForceUpdate = false;
              }
            }
            if (dataMap['forced_update'] is int){
              if (dataMap['forced_update'] == 1){
                isForceUpdate = true;
              }else{
                isForceUpdate = false;
              }
            }
          }
        }
      }

//      currentVersion = '1.0.0';
//      newVersion = '1.0.1';

      LogUtils.d('isForce====$isForceUpdate===oldVersion$currentVersion===newVersion====$newVersion');

      List currentArr = currentVersion.split('.');
      List appStoreArr = newVersion.split('.');

      bool isShow = false;
      bool isSH = false;
      if (currentArr.length == appStoreArr.length){
        if (int.parse(currentVersion.replaceAll('.', '')) < int.parse(newVersion.replaceAll('.', ''))){
          isShow = true;
        }

        if (int.parse(currentVersion.replaceAll('.', '')) > int.parse(newVersion.replaceAll('.', ''))){
          isSH = true;
        }else{
          isSH = false;
        }

      }else{

        if (currentArr.length <= 3 && appStoreArr.length <= 3){

          if (int.parse(currentArr[0]) < int.parse(appStoreArr[0])){
            isShow = true;
          }else if (int.parse(currentArr[0]) == int.parse(appStoreArr[0])){
            if (int.parse(currentArr[1]) < int.parse(appStoreArr[1])){
              isShow = true;
            }
          }
          if (int.parse(currentArr[0]) > int.parse(appStoreArr[0])){
            isSH = true;
          }else if (int.parse(currentArr[0]) == int.parse(appStoreArr[0])){
            if (int.parse(currentArr[1]) > int.parse(appStoreArr[1])){
              isSH = true;
            }else{
              isSH = false;
            }
          }else{
            isSH = false;
          }

        }else {
          if (int.parse(currentVersion.replaceAll('.', '')) < int.parse(newVersion.replaceAll('.', ''))){
            isShow = true;
          }
          if (int.parse(currentVersion.replaceAll('.', '')) > int.parse(newVersion.replaceAll('.', ''))){
            isSH = true;
          }else{
            isSH = false;
          }
        }

      }
      if (isShowTip == true){
        if (isShow == false){
//          KYToast.showText('当前已是最新版本～');
        }else{

          if (isAndroid){
            showDialog(
                barrierDismissible: false,//点击灰色背景的时候是否消失弹出框
                context: context,
                builder: (context) {
                  return new WillPopScope(
                    onWillPop: () async => false,
                    child: AlertDialog(
                      title: Text("温馨提示"),
                      content: Text("当前有新版本，是否前往更新"),
                      actions: <Widget>[
                        TextButton(

                          child: Text("取消"),
                          onPressed: () {
                            if (isForceUpdate == false){
                              Navigator.pop(context);
                            } else {
                              KYToast.showText('该版本比较重要，建议更新哦～');
                            }
                          },
                        ),
                        TextButton(
                          child: Text("更新"),
                          onPressed: () async{
                            if (isForceUpdate == false){
                              Navigator.pop(context);
                            }

                            Clipboard.setData(ClipboardData(text: andriodDownloadUrl));
                            if (await canLaunch(andriodDownloadUrl)) {
                              await launch(andriodDownloadUrl);
                            } else {
                              KYToast.showText('下载链接已复制，您也可以手动前往浏览器粘贴下载～');
                              throw 'Could not launch $andriodDownloadUrl';
                            }

                          },
                        )
                      ],
                    ),
                  );
                });

          }else{
            showDialog(
                barrierDismissible: false,//点击灰色背景的时候是否消失弹出框
                context: context,
                builder: (context) {
                  return new WillPopScope(
                    onWillPop: () async => false,
                    child: AlertDialog(
                      title: Text("温馨提示"),
                      content: Text("当前有新版本，是否前往更新"),
                      actions: <Widget>[
                        TextButton(

                          child: Text("取消"),
                          onPressed: () {
                            if (isForceUpdate == false){
                              Navigator.pop(context);
                            } else {
                              KYToast.showText('该版本比较重要，建议更新哦～');
                            }
                          },
                        ),
                        TextButton(
                          child: Text("更新"),
                          onPressed: () async{
                            if (isForceUpdate == false){
                              Navigator.pop(context);
                            }
                            Clipboard.setData(ClipboardData(text: iosDownloadUrl));
                            if (await canLaunch(iosDownloadUrl)) {
                              await launch(iosDownloadUrl);
                            } else {
                              KYToast.showText('下载链接已复制，您也可以手动前往浏览器粘贴下载～');
                              throw 'Could not launch $iosDownloadUrl';
                            }
                          },
                        )
                      ],
                    ),
                  );
                });
          }

        }
      }


      LogUtils.d('isSh====$isSH====isShow====$isShow');



    },fail: (code,msg){
      LogUtils.d('getNewVersionAndIsForceUpdate===$code');
    });

  }


}